----------------------------------------------------------------------------------
-- Company:  
-- Engineer: Kenneth ØStby
-- 
-- Create Date:    12:02:44 03/29/2007 
-- Design Name: 
-- Module Name:    cacheBlock - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
-- Important to notice that all data coming out of the microBlaze core
-- is bit reversed. Hence the index and tag of the address is switched
-- compared to the normal way of doing it.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity cacheBlock is
	generic( 
		DATA_WIDTH			: integer := 32;
		ADDRESS_WIDTH		: integer := 32;	--Tag length is independent of the SET_A 
		SET_ASSOCIATIVITY	: integer := 2;	-- be able to create different caches, not 
		TAG_LENGTH			: integer := 25;	-- just set associative ones.
		ADDR_LENGTH			: integer := 7+1;
		
		LRU_SIZE			: integer := 2
	);
	
	port (
		CLOCK				: IN std_logic;
		RESET				: IN std_logic;
		IN_OMA_VALID		: OUT std_logic;
		IN_OMA_DATA_I		: OUT std_logic_vector(0 to 31);
		IN_OMA_DATA_O		: IN std_logic_vector(0 to 31);
		IN_OMA_ADDRESS		: IN std_logic_vector(0 to 31);
		IN_OMA_BE			: IN std_logic_vector(0 to 3);
		IN_OMA_RW			: IN std_logic;
		IN_OMA_ACTIVE		: IN std_logic;
		
		OUT_OMA_VALID		: IN std_logic;
		OUT_OMA_DATA_I		: IN std_logic_vector(0 to 31);
		OUT_OMA_DATA_O		: OUT std_logic_vector(0 to 31);
		OUT_OMA_ADDRESS		: OUT std_logic_vector(0 to 31);
		OUT_OMA_BE			: OUT std_logic_vector(0 to 3);
		OUT_OMA_RW			: OUT std_logic;
		OUT_OMA_ACTIVE		: OUT std_logic
	);
end cacheBlock;

architecture Behavioral of cacheBlock is
	type dlines is array (0 to SET_ASSOCIATIVITY -1) of 
		std_logic_vector(0 to DATA_WIDTH-1);

	type STATE is ( IDLE, FIND_VICTIM, EVICT, GET_DATA, OVERRIDE, DONE );

	constant tagNotFound	: std_logic_vector(0 to SET_ASSOCIATIVITY -1) := (others => '0');

	-- Which parts shall we address, and which are left for tags.
	constant setAddrBase	: integer := ADDRESS_WIDTH - ADDR_LENGTH;
	constant lruBase		: integer := 0; -- Base addr of LRU;
	constant lruSize		: integer := 1; -- Log2(SET_ASSOCIATIVITY);
	constant dirtyBase		: integer := lruBase + lruSize;
	constant validBase		: integer := dirtyBase + 1;
	constant tagBase		: integer := validBase + 1;
	constant tagSize		: integer := setAddrBase;
	
	-- Control signals.
	signal enabled			: std_logic_vector(0 to SET_ASSOCIATIVITY - 1);
	signal writeEnabled		: std_logic_vector(0 to SET_ASSOCIATIVITY - 1);
	signal tagFound			: std_logic_vector(0 to SET_ASSOCIATIVITY - 1);

	-- Cache state
	signal readState 		: STATE := IDLE;
	signal writeState 		: STATE := IDLE;
	
	-- Set up the signals to be multiplexed.
	signal dout				: dlines := (others => (others => '0'));
	signal mout				: dlines := (others => (others => '0'));
	signal din				: dlines := (others => (others => '0'));
	signal min				: dlines := (others => (others => '0'));

	-- register the signal to meet timing constraints.
	signal dataIn			: std_logic_vector(0 to IN_OMA_DATA_O'length -1);

	-- BlockRAM needs delay.
	signal delay 			: std_logic_vector(0 to 1) := "00";
	signal inValid			: std_logic := '0';
	
	impure function findVictim return integer is
		variable minValue 	: integer := 90; --Change this if you have set associative > 90
		variable minPos		: integer := 0;
		variable current	: integer := 0;
	begin
		for I in 0 to SET_ASSOCIATIVITY -1 loop
			if mout(I)(validBase) = '0' then 
				return I;
			else
				current := conv_integer(mout(I)(lruBase to lruBase + lruSize - 1));
				if current < minValue then
					minValue := current;
					minPos := I;	
				end if;
			end if;
		end loop;

		return minPos;
	end function;

	function lruReduction(A:std_logic_vector) return integer is
		variable x : integer ;
	begin
		x := conv_integer(A);	
		if x >= 0 then
			return x;
		else return 0;
		end if;
	end function;
begin
	-- Generate the storage blocks.
	setBlock: for I in 0 to SET_ASSOCIATIVITY - 1 generate
	begin
		cacheBlock: entity work.myRam
			generic map(
				DATA_WIDTH 	=> DATA_WIDTH,
				ADDR_LENGTH => ADDR_LENGTH
			)	
			port map(
				clock			=> CLOCK,		
				reset			=> RESET,	
				enable			=> enabled(I),	
				writeEnabled	=> writeEnabled(I),
				address			=> IN_OMA_ADDRESS(IN_OMA_ADDRESS'length - ADDR_LENGTH to IN_OMA_ADDRESS'length -1 ),
				dataIn			=> din(I),		
				metaIn			=> min(I),
				dataOut			=> dout(I),
				metaOut			=> mout(I)	
			);
	end generate;

	-- Check if we've found something.
	-- update the tagFound signal.
	fnd: process(CLOCK,IN_OMA_ADDRESS,mout) is
	begin
		for I in 0 to SET_ASSOCIATIVITY -1 loop
			if (mout(I)(tagBase to tagBase + tagSize -1) = IN_OMA_ADDRESS(0 to IN_OMA_ADDRESS'length - ADDR_LENGTH- 1))
				and mout(I)(validBase) = '1'
			then 
				tagFound(I) <= '1';
			else 
				tagFound(I) <= '0';
			end if; 
		end loop;
	end process;

	-- The main function.
	main: process(CLOCK, IN_OMA_ACTIVE) is
		variable victim : natural := 0;

		-- Function decl.
		procedure updateLRU is
		begin
			for I in 0 to SET_ASSOCIATIVITY - 1 loop
				-- Copy back, rude and elementary. From the output to the input...
				min(I) <= mout(I);
				
				-- Update only the LRU.
				min(I)(lruBase to lruBase + lruSize -1) <= CONV_STD_LOGIC_VECTOR(
								lruReduction(mout(I)(lruBase to lruBase + lruSize -1)),
								lruSize);	
			end loop;
		end updateLRU;

		procedure evictLine is
		begin
			-- If victim dirty
			-- dump it to memory.
			OUT_OMA_BE			<= (others => '1');
			OUT_OMA_RW			<= '0';
			OUT_OMA_ACTIVE		<= '1';
			OUT_OMA_DATA_O		<= dout(victim);
			-- The address..

			OUT_OMA_ADDRESS(IN_OMA_ADDRESS'length - ADDR_LENGTH to IN_OMA_ADDRESS'length -1 ) <= 
				IN_OMA_ADDRESS( IN_OMA_ADDRESS'length - ADDR_LENGTH to IN_OMA_ADDRESS'length -1);			
			-- Get the tag from the metadata.
			OUT_OMA_ADDRESS(0 to OUT_OMA_ADDRESS'length - ADDR_LENGTH- 1 ) <= mout(victim)(tagBase to tagBase + tagSize - 1);
		end evictLine;

		procedure doWriteHit is
		begin
			updateLRU;
			min(victim)(validBase) <= '1';
			min(victim)(dirtyBase) <= '1';
			min(victim)(lruBase to lruSize -1) <= (others => '1');
			min(victim)(tagBase to tagBase + tagSize -1) <= mout(victim)(tagBase to tagBase+tagSize-1);
			din(victim) 			<= dataIn;
			writeEnabled(victim) 	<= '1';

			inValid <= '1';
		end doWriteHit;

		procedure doReadHit is
		begin
			updateLRU;
			for I in 0 to SET_ASSOCIATIVITY -1 loop
				if ( tagFound(I) = '1' ) then
					IN_OMA_DATA_I <= dout(I);
					exit;
				end if;
			end loop;

			inValid <= '1';
		end doReadHit;

		procedure doReadMiss is
		begin
			-- 1) Find victim
			-- 2) Evict
			-- 3) Get data from memory
			-- 4) Override
			case readState is 
				when IDLE =>
					readState <= FIND_VICTIM;
				when FIND_VICTIM =>
					victim := findVictim; 
					readState <= EVICT;
				when EVICT =>
					if OUT_OMA_VALID = '1' then
						OUT_OMA_ADDRESS	<= IN_OMA_ADDRESS;
						OUT_OMA_RW		<= '1';
						OUT_OMA_ACTIVE	<= '1';
						-- Send out a new request.
						readState <= GET_DATA;
					else 
						if  mout(victim)(dirtyBase) = '1' and
							mout(victim)(validBase)  = '1' then 

							-- If it's dirty evict it.	
							evictLine;	
						else 
							OUT_OMA_ADDRESS	<= IN_OMA_ADDRESS;
							OUT_OMA_RW		<= '1';
							OUT_OMA_ACTIVE	<= '1';
							readState <= GET_DATA;
						end if; 
					end if;

				when GET_DATA =>
					OUT_OMA_ACTIVE	<= '1';
					if ( OUT_OMA_VALID = '1' ) then 

						-- Update metadata.
						min(victim)(validBase)		<= '1';
						min(victim)(dirtyBase) 		<= '0';
						min(victim)(lruBase to lruSize -1) <= (others => '1');

						 min(victim)(tagBase to tagBase + tagSize -1) <= IN_OMA_ADDRESS(0 to IN_OMA_ADDRESS'length - ADDR_LENGTH -1);
						din(victim) 				<= OUT_OMA_DATA_I;

						-- Send stuff out.		
						IN_OMA_DATA_I 	<= OUT_OMA_DATA_I;
						
						-- writeEnabled asserted for two cycles.
						readState <= DONE;
						writeEnabled(victim) 		<= '1';
					end if; 	
				when DONE =>
					inValid	<= '1';
					writeEnabled(victim) 		<= '1';
					readState <= IDLE; 
					
				when others =>
					readState <= IDLE;
			end case;
		end doReadMiss;


		procedure doWriteMiss is
		begin
			--- 1) Find victim.
			--- 2) Evict. E.g., die bastard cache line
			--- 3) Override. 

			case writeState is 
				when IDLE =>
					writeState <= FIND_VICTIM;
				when FIND_VICTIM =>
					victim := findVictim; 
					writeState <= EVICT;
				when EVICT =>
					if OUT_OMA_VALID = '1' then
						writeState <= OVERRIDE;
					else
						if  mout(victim)(dirtyBase) = '1' and
							mout(victim)(validBase)  = '1' then 
							-- If it's dirty and valid evict it.	
							evictLine;	
						else 
							writeState <= OVERRIDE;
						end if; 
					end if;
				when OVERRIDE =>
					min(victim)(validBase) <= '1';
					min(victim)(dirtyBase) <= '1';
					min(victim)(lruBase to lruSize -1) <= (others => '1');
					min(victim)(tagBase to tagBase + tagSize -1) <= IN_OMA_ADDRESS(0 to IN_OMA_ADDRESS'length - ADDR_LENGTH -1);
					din(victim) 			<= dataIn;
					writeState <= DONE;

					-- writeEnabled asserted for two clock cycles.
					writeEnabled(victim)	<= '1';
					inValid <= '1';
				when DONE =>
					writeEnabled(victim)	<= '1';
					writeState <= IDLE;

				when others => 
					writeState <= IDLE;
			end case;
			
		end doWriteMiss;
		-- decl. ends.
	
	begin
		dataIn			<= IN_OMA_DATA_O;
		IN_OMA_VALID	<= inValid;
		
		if rising_edge(CLOCK)  then
			inValid 	<= '0';
			OUT_OMA_ACTIVE 	<= '0';

			writeEnabled <= (others => '0');
			enabled		<= (others => '0');

			if ( inValid = '0' )  then
				enabled <= (others => '1');
			else 
				enabled <= (others => '0');
			end if;

			if IN_OMA_ACTIVE = '1' and inValid = '0' then

				if IN_OMA_ADDRESS = x"7FFFFFFF" and IN_OMA_RW = '0' then
					-- DO NOT cache the termination signal.
					-- We're done anyways, so lets do this the 
					-- dirty way.
					OUT_OMA_ADDRESS <= IN_OMA_ADDRESS;
					OUT_OMA_RW		<= '0';
					OUT_OMA_ACTIVE	<= '1';
				else 
					-- We need to let the signals propagate from the block RAM to us. 
					if delay = "11" or ( writeState /= IDLE or readState /= IDLE ) then 
						if IN_OMA_RW = '1' then
							if tagFound /= tagNotFound then
								-- Ladies and Gentlemen, we have hit.
								doReadHit;
							else
								doReadMiss;
							end if;
						elsif IN_OMA_RW = '0' then
							if tagFound /= tagNotFound then
								doWriteHit;
							else
								doWriteMiss;
							end if;
						end if;

						delay <= "00";
					elsif delay = "00" then
						-- Block RAM need time to propagate through...
						delay <= "01";
					else 
						delay <= "11";
					end if; 

				end if; 
			end if;
		end if;
	end process;

end Behavioral;

