#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "controller.h"
#include "cpu.h"
#include "memory.h"
#include "util.h"


//  #define DEBUG

static Memory *m;
static STATE state;

int run = 1; 

unsigned long long int memRead = 0, memWrite = 0; 

unsigned char byteEnabled = 0;

__inline__ unsigned long long int rdtsc() {
	unsigned long long int x;
	__asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
	return x;
}


void termi(int code){
	printf("\nNow exiting with code: %d\n", code);

#ifdef DEBUG
	printf("Want to display a memory map?\n");
	if ( getchar() == 'y' ){
		printf("Memory Map\n");
		mem_print(m);
	}
#endif 

	printf("\n");
	exit(code);
}

void trap_seg(int signal){
	printf("Seg.fault\n");
	termi(5);
}

int handle(unsigned int command){
	unsigned int reversed = reverseInt(command);
	
#ifdef DEBUG
	printf("Doing state: 0x%08x,r: 0x%08x\n",command,reversed);
#endif

	if( state == IDLE ) {
#ifdef DEBUG
		printf("Reverse: 0x%08X\n", reversed); 
#endif
		if( reversed == READ_SIGNAL ){
			state = R_ADDR;
		} else if ( (reversed & 0xFF) == 0x02 ){
			state = W_ADDR;
			// Byte Enabled Hack.
			byteEnabled = (reversed >> 4*3) & 0xF;
#ifdef DEBUG
			printf("Byte Lines: 0x%08X\n",byteEnabled);
#endif
		}
	}else if( WRITING <= state && state <= W_DATA){
		state = handleWrite(command,state);
	}else if( READING <= state && state <= R_DATA) {
		state = handleRead(command,state);
	}else if ( state == COMPLETE ){
#ifdef DEBUG
		printf("Operation took 0x%08X operations\n",reversed);
#endif
		state = IDLE;
	}

}

int handleRead(unsigned int command,const STATE state){
	/* The FPGA tries to access a memory location */
	unsigned int reversed = reverseInt(command);
	unsigned int r; 
	
#ifdef DEBUG
	printf("Now handling read to memory: 0x%X\n",reversed);
#endif
	int i = mem_read(reversed,NULL,m);
	r = reverseInt(i);	

#ifdef DEBUG
	printf("Sending value: 0x%08x, r=0x%08x\n",i,r);
#endif
	writeWord32(i);	
	memRead++;

	// IDLE STAGE HACK!
	getWord32();
	
	return COMPLETE;
}

int handleWrite(unsigned int command,const STATE state){
	unsigned int reversed = reverseInt(command);
	/* The FPGA tries to write to a memory location */

	unsigned int i = 0;
	
	switch( state ) {
		case W_ADDR:
			i = getWord32();
#ifdef DEBUG
			printf("Storing to mem: 0x%08x, data: 0x%08x\n",reversed,i);
#endif
			if ( reversed == 0x7FFFFFFF ) {
				printf("\nTerminate signal from CPU\n");
				run = 0 ;
			//	termi(2);
			}
			memWrite++;

			if ( run ) mem_write(reversed,i ,byteEnabled, m);
		default: return COMPLETE;
	}

}

print_state() {
	switch(state) {
		case IDLE:
			printf("State: IDLE\n");
			break;
		case WRITING:
			printf("State: WRITING\n");
			break;
		case W_ADDR:
			printf("State: W_ADDR\n");
			break;
		case W_DATA:
			printf("State: W_DATA\n");
			break;
		case READING:
			printf("State: READING\n");
			break;
		case R_ADDR:
			printf("State: R_ADDR\n");
			break;
		case R_DATA:
			printf("State: R_DATA\n");
			break;
		case COMPLETE:
			printf("State: COMPLETE\n");
			break;
		default: printf("State: UNKNOWN\n");
	}
}

int main(int argc, char **argv) {
	MemBlock b;
	int f = 0; 

	if( argc < 2 ) {
		printf("Usage: %s <bitfile> [application]\n", argv[0]);
		exit(1);
	}

// 	signal(SIGSEGV,trap_seg);

	// Init the memory
	m = mem_init();
	int mm;
	if ( argc > 2 ){
		if ( (mm = mem_load(m,argv[2] ) < 0 )){
			printf("Error opening file: %s, error: %d\n", argv[2], mm );
			exit(1);
		}
	}

	state = IDLE;

#ifdef DEBUG
	printf("Now opening the card\n");
#endif
	f = openCard(1, argv[1]);
#ifdef DEBUG
	printf("OpenCard returned: %d\n\n",f);
#endif

	//================== Configuration done.
	unsigned long long int start, end, runtime;
	start = end = 0 ;
	
	f = '\0';

	start = rdtsc();
	do {
#ifdef DEBUG
		print_state();
#endif

		switch(f){
			case 's':
#ifdef DEBUG
				printf("Now writing\n");
#endif
				writeWord32(0xFF);				
				break;
			case 'r':
#ifdef DEBUG
				printf("Reset\n");
#endif
				break;
			default:
				break;
		}
		
		handle(getWord32());
#ifdef DEBUG
		printf("Press q to quit, everything else to continue\n");
		getchar();
#endif
	}while ( run );
	end = rdtsc();
	
	printf("Stats:\n");
	printf("\tRunning time: %llu\n", end - start);
	printf("\tTotal mem access: %llu\n", memRead + memWrite);
	printf("\tTotal mem reads: %llu\n", memRead );
	printf("\tTotal mem writes: %llu\n", memWrite );

	printf("Did you find anything?\nNow exiting\n");
	f = closeCard();
	printf("Close card returned: %d\n",f);
	printf("Now printing memory\n");
	
	for(f=0; f<m->mem_used;f++){
		b = m->data[f];
		printf("Cell %d contains %02x\n",b.addr,b.data);
	}
	
	termi(3);

	return 1;
}

